<?php

use GuzzleHttp\Client;
use GuzzleHttp\Promise\PromiseInterface;

class Device extends Entity
{
    public $androidID;

    public $name;

    public $token;

    public $model;

    public $androidVersion;

    public $appVersion;

    public $userID;

    public $enabled;

    public $user;

    public static $relations = [
        "User" => ["ID", "userID"]
    ];

    public function __construct()
    {
        $this->user = new User();
    }

    /**
     * @return string
     */
    public function getUserID()
    {
        return $this->userID;
    }

    /**
     * @param string $userID
     */
    public function setUserID($userID)
    {
        $this->userID = $userID;
    }

    /**
     * @return string
     */
    public function getModel()
    {
        return $this->model;
    }

    /**
     * @param string $model
     */
    public function setModel($model)
    {
        $this->model = $model;
    }

    /**
     * @return string
     */
    public function getAndroidVersion()
    {
        return $this->androidVersion;
    }

    /**
     * @param string $androidVersion
     */
    public function setAndroidVersion($androidVersion)
    {
        $this->androidVersion = $androidVersion;
    }

    /**
     * @return string
     */
    public function getAppVersion()
    {
        return $this->appVersion;
    }

    /**
     * @param string $appVersion
     */
    public function setAppVersion($appVersion)
    {
        $this->appVersion = $appVersion;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $name
     */
    public function setName($name)
    {
        $this->name = $name;
    }

    /**
     * @return string
     */
    public function getToken()
    {
        return $this->token;
    }

    /**
     * @param string $token
     */
    public function setToken($token)
    {
        $this->token = $token;
    }

    /**
     * @return string
     */
    public function getAndroidID()
    {
        return $this->androidID;
    }

    /**
     * @param string $androidID
     */
    public function setAndroidID($androidID)
    {
        $this->androidID = $androidID;
    }

    /**
     * @return int
     */
    public function getEnabled()
    {
        return $this->enabled;
    }

    /**
     * @param int $enabled
     * @throws Exception
     */
    public function setEnabled($enabled)
    {
        if (isset($this->ID) && $enabled === 0) {
            $this->unsetPrimaryDevice();
        }
        $this->enabled = $enabled;
    }

    /**
     * @return User
     */
    public function getUser()
    {
        return $this->user;
    }

    /**
     * @param User $user
     */
    public function setUser($user)
    {
        $this->user = $user;
    }

    public function delete()
    {
        $this->unsetPrimaryDevice();
        parent::delete();
    }

    public function __toString()
    {
        return empty($this->getName()) ? "{$this->getModel()} [{$this->getID()}]" : "{$this->getName()} [{$this->getID()}]";
    }

    /**
     * @param array $sims
     * @throws Exception
     */
    public function saveSims(array $sims)
    {
        if (!empty($sims)) {
            foreach ($sims as $sim) {
                $obj = new Sim();
                $obj->setSlot($sim->slot);
                $obj->setDeviceID($this->getID());
                if ($obj->read(false)) {
                    $sim->ID = $obj->getID();
                } else if (!$sim->enabled) {
                    continue;
                }
                $sim = Sim::fromObject($sim);
                $sim->setDeviceID($this->getID());
                $sim->save(false);
            }
        }
    }

    /**
     * @param bool $save
     * @throws Exception
     */
    public function unsetPrimaryDevice()
    {
        if ($this->getUser()->getPrimaryDeviceID() == $this->getID()) {
            $devices = $this->getUser()->getDevices();
            if (count($devices) > 1) {
                foreach ($devices as $device) {
                    if ($device->getID() != $this->getID()) {
                        $this->getUser()->setPrimaryDeviceID($device->getID());
                        break;
                    }
                }
            } else {
                $this->getUser()->setPrimaryDeviceID(0);
            }
            $this->getUser()->save();
        }
    }

    /**
     * @param array $data
     * @return PromiseInterface
     */
    public function sendPushNotification($data = array())
    {
        $url = 'https://fcm.googleapis.com/fcm/send';
        $fields = array('to' => $this->getToken(), 'priority' => 'high');
        if (count($data) > 0) {
            $fields["data"] = $data;
        } else {
            $fields["dry_run"] = true;
        }

        $headers = [
            'Authorization' => 'key=' . SERVER_KEY
        ];
        $client = new Client();
        return $client->postAsync($url, ['json' => $fields, 'headers' => $headers, 'verify' => false]);
    }

    /**
     * @param array $groups
     * @param bool $countCredits
     * @throws Exception
     */
    public static function processRequests($groups, $countCredits = false)
    {
        $promises = [];
        foreach ($groups as $deviceID => $group) {
            // Initiate each request but do not block
            $promises[$deviceID] = $group["device"]->sendPushNotification($group["data"]);
        }

        // Wait for the requests to complete, even if some of them fail
        $results = GuzzleHttp\Promise\settle($promises)->wait();

        $errors = [];
        foreach ($results as $deviceID => $res) {

            /** @var Device $device */
            $device = $groups[$deviceID]["device"];

            if ($res["state"] == PromiseInterface::FULFILLED) {
                /** @var GuzzleHttp\Psr7\Response $response */
                $response = $res["value"];

                if ($response->getStatusCode() === 200) {
                    $result = json_decode($response->getBody(), true);
                    if ($result["success"] != '1') {
                        $error = $result["results"][0]["error"];
                        if ($error === 'NotRegistered' || $error === 'InvalidRegistration') {
                            $device->setEnabled(0);
                            $device->save();
                        }
                        $errors[] = strval($device) . " : {$error}";
                    } else if ($countCredits) {
                        if (!is_null($device->getUser()->getCredits())) {
                            $count = $groups[$deviceID]["count"];
                            MysqliDb::getInstance()->rawQuery("UPDATE User SET credits = credits - {$count} WHERE ID = {$device->getUserID()}");
                        }
                    }
                } else {
                    $errors[] = strval($device) . " : {$response->getStatusCode()} {$response->getReasonPhrase()}";
                }
            } else {
                $errors[] = strval($device) . " : {$res["reason"]}";
            }
        }

        if (count($errors) > 0) {
            $error = "<p>You have following errors while sending messages.</p>";
            $error .= "<ul>";
            foreach ($errors as $err) {
                $error .= "<li>{$err}</li>";
            }
            $error .= "</ul>";
            throw new Exception($error);
        }
    }

    /**
     * @param int $id
     * @param int $userID
     * @return bool|Device
     * @throws Exception
     */
    public static function getById($id, $userID)
    {
        $currentDevice = new Device();
        $currentDevice->setID($id);
        $currentDevice->setUserID($userID);
        return $currentDevice->read();
    }

    /**
     * @param string $string
     * @return bool|array
     */
    public static function isValidIdentifier($string)
    {
        $identifiers = explode('|', $string);
        if (count($identifiers) > 2) {
            return false;
        }
        foreach ($identifiers as $identifier) {
            if (!ctype_digit($identifier)) {
                return false;
            }
        }
        return $identifiers;
    }
}